package compbio.ws.server;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Date;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import javax.annotation.Resource;
import javax.jws.WebService;
import javax.servlet.http.HttpServletRequest;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import org.apache.log4j.Logger;

import compbio.data.msa.JABAService;
import compbio.ws.client.Services;
import compbio.ws.client.WSTester;

/**
 * JABAWS services registry
 * 
 * @author pvtroshin
 * 
 */
@WebService(endpointInterface = "compbio.data.msa.RegistryWS", targetNamespace = "http://msa.data.compbio/01/12/2010/", serviceName = "RegistryWS")
public class RegistryWS implements compbio.data.msa.RegistryWS, JABAService {

	// Ask for resource injection
	@Resource
	WebServiceContext wsContext;

	private static Logger log = Logger.getLogger(RegistryWS.class);

	private final static Map<Services, Date> operating = new ConcurrentHashMap<Services, Date>();

	@Override
	public Set<Services> getSupportedServices() {
		init();
		return operating.keySet();
	}

	private void init() {
		// This should not be run concurrently
		if (operating.isEmpty()) {
			synchronized (operating) {
				if (operating.isEmpty()) {
					testAllServices();
				}
			}
		}
	}

	@Override
	public int getLastTested(Services service) {
		Date testedOn = getLastTestedOn(service);
		if (testedOn != null) {
			return (int) ((System.currentTimeMillis() - testedOn.getTime()) / 1000);
		}
		return 0;
	}

	/**
	 * Can potentially return null if the service has not been tested yet.
	 */
	@Override
	public Date getLastTestedOn(Services service) {
		if (operating.containsKey(service)) {
			return operating.get(service);
		}
		return null;
	}

	/**
	 * TODO improve reporting. stop testing service on unsupported runtime env
	 * exception
	 */
	@Override
	public String testAllServices() {
		Writer testlog = new StringWriter();
		PrintWriter writer = new PrintWriter(testlog, true);
		WSTester tester = new WSTester(getServicePath(), writer);
		// This is done deliberately to prevent malicious user from overloading
		// the server
		synchronized (operating) {
			for (Services service : Services.values()) {
				if (tester.checkService(service)) {
					operating.put(service, new Date());
				}
			}
		}
		writer.close();
		return testlog.toString();
	}

	private String getServicePath() {
		assert wsContext != null : "WS context injection failed!";
		MessageContext msContext = wsContext.getMessageContext();
		HttpServletRequest request = (HttpServletRequest) msContext
				.get(MessageContext.SERVLET_REQUEST);

		StringBuffer server = request.getRequestURL();
		server = server.delete(server.lastIndexOf("/"), server.length());
		return server.toString();
	}

	@Override
	public String testService(Services service) {
		String server = getServicePath();
		Writer testlog = new StringWriter();
		PrintWriter writer = new PrintWriter(testlog, true);
		WSTester tester = new WSTester(server, writer);
		try {
			synchronized (operating) {
				boolean succeed = tester.checkService(service);
				if (succeed) {
					operating.put(service, new Date());
				}
			}
		} finally {
			writer.close();
		}
		return testlog.toString();
	}

	@Override
	public boolean isOperating(Services service) {
		init();
		return operating.containsKey(service);
	}

}
